@@ -1,5 +1,5 @@
 package IO::Pager::Perl;
-our $VERSION = '2.01'; #Untouched since 2.01
+our $VERSION = '2.10'; #Untouched since 2.10
 
 use strict;
 use warnings;
@@ -14,11 +14,10 @@ sub ReadMode;
 sub ReadKey;
 
 sub new {
-  print "\e[?1000;1006;1015h";
   my $class = shift;
   my %param = @_;
-  local $ENV{TERM} = $ENV{TERM} || '';
-  local $ENV{TERMCAP} = $ENV{TERM} || '';
+  $ENV{TERM} = $ENV{TERM} || '';
+  $ENV{TERMCAP} = $ENV{TERMCAP} || '';
 
   my %dims = get_size(cols =>$param{cols} ||80,
 		      rows =>$param{rows} ||25,
@@ -27,25 +26,38 @@ sub new {
 
   #screen is vt100 compatible but does not list sf?!
   #No matter, it's only used for workaround mode.
-  if( $ENV{TERM} eq 'screen' && $ENV{TERMCAP} !~ /sf/ ){
-    $ENV{TERM} = 'vt100';
+  $ENV{TERM} = 'vt100' if( $ENV{TERM} eq 'screen' && $ENV{TERMCAP} !~ /sf/ );
+
+  #Hack together Windows support. We could use Term::Screen(::Uni),
+  #but that uses many layers of tie-ing, some of which could be inheritiance.
+  #This way also reduces dependencies
+  if( $^O =~ /MSWin/ ){
+    eval "use Win32::Console::ANSI;";
+    if( $@ ){
+      warn "Could not load Win32::Console::ANSI, falling back to dumb mode - $@"; }
+    else{
+      $ENV{TERM} = 'WINANSI';
+      #Windows lacks vb as does the fallback Term::Cap vt220 entry, add our own
+      #https://www.ibiblio.org/oswg/oswg-nightly/oswg/en_US.ISO_8859-1/articles/alessandro-rubini/visual-bell/visual-bell-howto.html#VISIBLEBELL
+      $ENV{TERMCAP} = do{ undef $/; $_=<DATA>; y/\n//d; $_ };
+    }
   }
-
-#cm=>cup, ce=>el, cl=>clear, sf=>ind, sr=>ri
-#md=>bold, me=>sgr0, mr=>rev, us=>smul
-  #Speed is mostly useless except Term::Cap expects it?
+  else{
+    #Try to enable mouse support
+    print "\e[?1000;1006;1015h";
+  }
+  #Speed is mostly useless except Term::Cap demands it
   my $t = Term::Cap->Tgetent({ OSPEED => $param{speed} });
-  eval{ $t->Trequire(qw/cm ce cl sf sr/) };
-  my $dumb = $@ ? 1 : 0;
+  my $dumb = eval{ $t->Trequire(qw/cm ce cl sf sr/) } ? 1 : 0;
 
-  my %primitives = (
-		    # if the entries don't exist, nothing bad will happen
+  #CORE:  cm=>cup,  ce=>el,   cl=>clear, sf=>ind,  sr=>ri
+  #EXTRA: md=>bold, me=>sgr0, mr=>rev,   us=>smul, vb=>flash
+  my %primitives = (# if the entries don't exist, nothing bad will happen
 		    BLD   => $t->Tputs('md'), # Bold
 		    ULN   => $t->Tputs('us'), # Underscore
 		    REV   => $t->Tputs('mr'), # Reverse
 		    NOR   => $t->Tputs('me'), # Normal
 		   );
-
   my $text;
   if( defined( $param{text} ) ){
     my $ref = ref( $param{text} );
@@ -126,12 +138,14 @@ EOH
   add_keys(\&to_bott,   '>', 'G', '$', "\e>", "\e[F", "\e0E", "\e0W", "\e[4~");
                                      #M->      ?     End     End     End
   add_keys(\&downpage,  ' ', 'z', "\cV", , 'f', "\cF", "\e ", "\e[6~"); #M-  PgDn
+  add_keys(\&downpage,  "\eOs") if $ENV{TERM} eq 'WINANSI';
   add_keys(\&downhalf,  'd', "\cD");
   add_keys(\&downline,  'e', 'j', 'J', "\cE", "\cN", "\e[B"); #down
-  add_keys(\&downline_raw, "\n");
+  add_keys(\&downline_raw, "\n", "\r");
   add_keys(\&upline,    'y', 'k', "\cY", "\cK", 'K', 'Y', "\cP", "\e[A"); #up
   add_keys(\&uphalf,    'u', "\cU");
   add_keys(\&uppage,    'w', 'b', "\cB", "\ev", "\e[5~"); #M-v PgUp
+  add_keys(\&uppage,  "\eOy") if $ENV{TERM} eq 'WINANSI';
   add_keys(\&to_top,    '<', 'g',      "\e<", "\e[H", "\e0",          "\e[1~");
                                         #M-<    Home    Home            Home
   add_keys(\&next_file, ':n', "\e[1;4C");
@@ -173,7 +187,7 @@ EOH
 
   $me->{_end} = $me->{rows} - 1;
 
-  $SIG{WINCH} = sub{ $me->resize() };
+  $SIG{WINCH} = sub{ $me->resize() } unless $ENV{TERM} eq 'WINANSI';
   $me->{cols}-- if $me->{scrollBar};
 
   #Can we fold?
@@ -207,7 +221,7 @@ sub resize {
   }
   $me->status();
 
-  $me->{WINCH}->() if ref($me->{WINCH}) eq 'CODE';  
+  $me->{WINCH}->() if ref($me->{WINCH}) eq 'CODE';
 }
 
 sub get_size {
@@ -230,13 +244,16 @@ sub get_size {
     *ReadKey = sub{ getc() };
 
     #Can we get better defaults?
-    if( `stty` =~ /speed/ ){
-      @dims{'rows','cols'} = ($1-1,$2-1) if `stty size` =~ /^(\d+)\s+(\d+)$/;
-      $dims{speed} = $1 if `stty speed` =~ /^(\d+)$/;
+    if( $ENV{TERM} eq 'WINANSI' ){
+	eval{ @dims{'rows','cols'} = Win32::Console::ANSI::Cursor() };
+    }
+    elsif( `stty` =~ /speed/ ){
+	@dims{'rows','cols'} = ($1-1,$2-1) if `stty size` =~ /^(\d+)\s+(\d+)$/;
+	$dims{speed} = $1 if `stty speed` =~ /^(\d+)$/;
     }
     else{
-      $dims{rows} = `tput lines`  || $dims{rows};
-      $dims{cols} = `tput cols`   || $dims{cols};
+	$dims{rows} = `tput lines`  || $dims{rows};
+	$dims{cols} = `tput cols`   || $dims{cols};
     }
   }
   return %dims;
@@ -351,7 +368,6 @@ sub more {
   }
   else{
     print $me->{NOR};
-    $DB::single = 1;
 
 
     #INPUT LOOP, revised with inspiration from Term::Screen::getch()
@@ -400,7 +416,7 @@ sub more {
 	elsif( $char eq "\cG" ){
 	  $input = '';
 	  $exit = 1; }
-	elsif( $char eq "\n" ){
+	elsif( $char eq "\n" || $char eq "\r" ){
 	  #Remove extraneous characters that could cause infinite error loop
 	  #XXX this prevents goofy RPN-like repeated commands
 	  $input =~ y/0-9//cd;#	  chomp($input);
@@ -774,6 +790,7 @@ sub down_lines{
       $me->{_cursor}++;
     }
   }
+  $me->refresh() if $ENV{TERM} eq 'WINANSI'; #XXX Windows scroll is lame
   $me->scrollBar() if $me->{scrollBar};
 }
 sub downhalf {  $_[0]->down_lines( $_[0]->{rows} / 2 ); }
@@ -781,7 +798,6 @@ sub downpage {  $_[0]->down_lines( $_[0]->{rows} );
 		#WTF?! add_text in tp's while-loop cannot be reached if there's
 		#no delay here until something other than downpage is called?!
 		select(undef, undef, undef, .1); #XXX WTF?!
-#		warn "\n\n", map{$_[0]->{$_}."\n"} qw/rows _cursor _end/;
 }
 sub downline {  $_[0]->down_lines( 1 ); }
 #Term::ReadKey doesn't offer sufficiently fine control; we want CS8 but -OCRNL
@@ -816,6 +832,7 @@ sub up_lines{
     }
   }
 
+  $me->refresh() if $ENV{TERM} eq 'WINANSI'; #XXX Windows scroll is lame
   print $me->{_term}->Tgoto('cm',0,$me->{rows});		# goto bottom
   $me->scrollBar() if $me->{scrollBar};
 }
@@ -1094,6 +1111,24 @@ sub dumb_mode{
 }
 
 1;
+__DATA__
+WINANSI|vt220|Win32 Console based on DEC VT220 in vt100 emulation mode:
+am:mi:xn:xo:
+co#80:li#24:
+RA=\E[?7l:SA=\E[?7h:
+ac=kkllmmjjnnwwqquuttvvxx:ae=\E(B:al=\E[L:as=\E(0:
+bl=^G:cd=\E[J:ce=\E[K:cl=\E[H\E[2J:cm=\E[%i%d;%dH:
+cr=^M:cs=\E[%i%d;%dr:dc=\E[P:dl=\E[M:do=\E[B:
+ei=\E[4l:ho=\E[H:im=\E[4h:
+is=\E[1;24r\E[24;1H:
+nd=\E[C:
+kd=\E[B::kl=\E[D:kr=\E[C:ku=\E[A:le=^H:
+mb=\E[5m:md=\E[1m:me=\E[m:mr=\E[7m:
+kb=\0177:
+r2=\E>\E[24;1H\E[?3l\E[?4l\E[?5l\E[?7h\E[?8h\E=:rc=\E8:
+sc=\E7:se=\E[27m:sf=\ED:so=\E[7m:sr=\EM:ta=^I:
+ue=\E[24m:up=\E[A:us=\E[4m:ve=\E[?25h:vi=\E[?25l:
+vb=\E7\E[?5h\E[?5l\E[?5h\E[?5l\E[?5h\E[?5l\E[?5h\E[?5l\E8:
 __END__
 =pod
 
@@ -1457,6 +1492,10 @@ only way it can effectively detect and accommodate changes in terminal size.
 If you also need notification of this signal, the handler will trigger any
 callback assigned to the I<WINCH> attribute of the C<new> method.
 
+I<WINCH> is not available on Windows. You will need to manually refresh your
+screen B<^L> if you resize the terminal in Windows to clean up the text
+however, this will not change the size of the pager itself.
+
 =head1 ENVIRONMENT
 
 IO::Pager::Perl checks the I<TERM> and I<TERMCAP> variables.
